@@ -22,8 +22,9 @@ import warnings
 from pkg_resources import iter_entry_points, EntryPoint
 
 from pluginlib.exceptions import PluginImportError, EntryPointWarning
+from pluginlib._objects import BlacklistEntry
 from pluginlib._parent import get_plugins
-from pluginlib._util import BlacklistEntry, LOGGER, NoneType, raise_with_traceback, BASESTRING
+from pluginlib._util import BASESTRING, LOGGER, NoneType, raise_with_traceback
 
 
 def format_exception(etype, value, tback, limit=None):
@@ -136,6 +137,7 @@ def _recursive_import(package):
             _import_module(submod[1], submod[0].path)
 
 
+# pylint: disable=too-many-instance-attributes,too-many-arguments
 class PluginLoader(object):
     """
     Args:
@@ -146,6 +148,7 @@ class PluginLoader(object):
         entry_point(str): `Entry point`_ for additional plugins
         blacklist(list): Iterable of :py:class:`BlacklistEntry` objects or tuples
         prefix_package(str): Alternative prefix for imported packages
+        type_filter(list): Iterable of parent plugin types to allow
 
     **Interface for importing and accessing plugins**
 
@@ -178,18 +181,22 @@ class PluginLoader(object):
     specified in ``paths``. Because the package paths will be traversed recursively, this should
     be an empty path.
 
+    ``type_filter`` limits plugins types to only those specified. A specified type is not
+    guaranteed to be available.
+
     .. _Entry point: https://packaging.python.org/specifications/entry-points/
     """
 
     def __init__(self, group=None, library=None, modules=None, paths=None, entry_point=None,
-                 blacklist=None, prefix_package='pluginlib.importer'):
+                 blacklist=None, prefix_package='pluginlib.importer', type_filter=None):
 
         # Make sure we got iterables
-        for argname, arg in (('modules', modules), ('paths', paths), ('blacklist', blacklist)):
+        for argname, arg in (('modules', modules), ('paths', paths), ('blacklist', blacklist),
+                             ('type_filter', type_filter)):
             if not isinstance(arg, (NoneType, Iterable)) or isinstance(arg, BASESTRING):
                 raise TypeError("Expecting iterable for '%s', recieved %s" % (argname, type(arg)))
 
-        # Make sure we got stings
+        # Make sure we got strings
         for argname, arg in (('library', library), ('entry_point', entry_point),
                              ('prefix_package', prefix_package)):
             if not isinstance(arg, (NoneType, BASESTRING)):
@@ -201,6 +208,7 @@ class PluginLoader(object):
         self.paths = paths or tuple()
         self.entry_point = entry_point
         self.prefix_package = prefix_package
+        self.type_filter = type_filter
         self.loaded = False
 
         if blacklist:
@@ -229,7 +237,7 @@ class PluginLoader(object):
         args = []
         for attr, default in (('group', '_default'), ('library', None), ('modules', None),
                               ('paths', None), ('entry_point', None), ('blacklist', None),
-                              ('prefix_package', 'pluginlib.importer')):
+                              ('prefix_package', 'pluginlib.importer'), ('type_filter', None)):
 
             val = getattr(self, attr)
             if default and val == default:
@@ -314,7 +322,7 @@ class PluginLoader(object):
 
         self.loaded = True
 
-    @ property
+    @property
     def plugins(self):
         """
         Newest version of all plugins in the group filtered by ``blacklist``
@@ -334,4 +342,54 @@ class PluginLoader(object):
             self.load_modules()
 
         # pylint: disable=protected-access
-        return get_plugins()[self.group]._newest(blacklist=self.blacklist)
+        return get_plugins()[self.group]._filter(blacklist=self.blacklist, newest_only=True,
+                                                 type_filter=self.type_filter)
+
+    @property
+    def plugins_all(self):
+        """
+        All resulting versions of all plugins in the group filtered by ``blacklist``
+
+        Returns:
+            dict: Nested dictionary of plugins accessible through dot-notation.
+
+        Similar to :py:attr:`plugins`, but lowest level is a regular dictionary of
+        all unfiltered plugin versions for the given plugin type and name.
+
+        Parent types are always included.
+        Child plugins will only be included if at least one valid, non-blacklisted plugin
+        is available.
+        """
+
+        if not self.loaded:
+            self.load_modules()
+
+        # pylint: disable=protected-access
+        return get_plugins()[self.group]._filter(blacklist=self.blacklist,
+                                                 type_filter=self.type_filter)
+
+    def get_plugin(self, plugin_type, name, version=None):
+        """
+        Args:
+            plugin_type(str): Parent type
+            name(str): Plugin name
+            version(str): Plugin version
+
+        Returns:
+            :py:class:`Plugin`: Plugin, or :py:data:`None` if plugin can't be found
+
+        Retrieve a specific plugin. ``blacklist`` and ``type_filter`` still apply.
+
+        If ``version`` is not specified, the newest available version is returned.
+        """
+
+        if not self.loaded:
+            self.load_modules()
+
+        # pylint: disable=protected-access
+        return get_plugins()[self.group]._filter(blacklist=self.blacklist,
+                                                 newest_only=True,
+                                                 type_filter=self.type_filter,
+                                                 type=plugin_type,
+                                                 name=name,
+                                                 version=version)
