@@ -40,7 +40,7 @@ class TestPluginLoaderInit(TestCase):
         """Error is raised when argument type is wrong"""
 
         # Expect iterables
-        for arg in ('modules', 'paths', 'blacklist'):
+        for arg in ('modules', 'paths', 'blacklist', 'type_filter'):
 
             with self.assertRaises(TypeError):
                 loader.PluginLoader(**{arg: 'string'})
@@ -113,7 +113,7 @@ def unload(module):
         del sys.modules[mod]
 
 
-# pylint: disable=protected-access
+# pylint: disable=protected-access,too-many-public-methods
 class TestPluginLoader(TestCase):
     """Tests for PluginLoader"""
 
@@ -306,6 +306,17 @@ class TestPluginLoader(TestCase):
 
         self.assertEqual(plugins.engine.steam.__module__, 'tests.testdata.importer.steam')
 
+    def test_type_filter(self):
+        """Filter plugin types"""
+
+        ploader = loader.PluginLoader(group='testdata', library='tests.testdata.lib',
+                                      type_filter=('engine', 'parser'))
+        plugins = ploader.plugins
+
+        self.assertTrue('parser' in plugins)
+        self.assertTrue('engine' in plugins)
+        self.assertFalse('hook' in plugins)
+
     def test_load_paths_missing(self):
         """Log on invalid path"""
 
@@ -372,6 +383,27 @@ class TestPluginLoader(TestCase):
 
             self.assertEqual(plugins1, plugins2)
 
+    def test_plugins_all(self):
+        """plugins only loads modules on the first call"""
+
+        ploader = loader.PluginLoader(group='testdata', library='tests.testdata.lib')
+        with mock.patch.object(ploader, 'load_modules',
+                               wraps=ploader.load_modules) as mock_load_modules:
+            plugins1 = ploader.plugins_all
+            self.assertEqual(mock_load_modules.call_count, 1)
+
+            self.assertEqual(len(plugins1.parser), 2)
+            self.assertTrue('xml' in plugins1.parser)
+            self.assertIsInstance(plugins1.parser.xml, dict)
+            self.assertTrue('json' in plugins1.parser)
+            self.assertIsInstance(plugins1.parser.json, dict)
+            self.assertEqual(sorted(plugins1.parser.json.keys()), ['1.0', '2.0'])
+
+            plugins2 = ploader.plugins_all
+            self.assertEqual(mock_load_modules.call_count, 1)
+
+            self.assertEqual(plugins1, plugins2)
+
     def test_blacklist(self):
         """Blacklist prevents listing plugin"""
 
@@ -381,3 +413,57 @@ class TestPluginLoader(TestCase):
 
         self.assertEqual(len(plugins.engine), 0)
         self.assertEqual(plugins.parser.json.version, '1.0')
+
+    def test_get_plugin(self):
+        """Retrieve specific plugin"""
+
+        ploader = loader.PluginLoader(group='testdata', library='tests.testdata.lib')
+        with mock.patch.object(ploader, 'load_modules',
+                               wraps=ploader.load_modules) as mock_load_modules:
+
+            jsonplugin = ploader.get_plugin('parser', 'json')
+            self.assertEqual(jsonplugin.name, 'json')
+            self.assertEqual(jsonplugin.version, '2.0')
+            self.assertEqual(mock_load_modules.call_count, 1)
+
+            jsonplugin = ploader.get_plugin('parser', 'json', '1.0')
+            self.assertEqual(jsonplugin.name, 'json')
+            self.assertEqual(jsonplugin.version, '1.0')
+            self.assertEqual(mock_load_modules.call_count, 1)
+
+    def test_get_plugin_missing(self):
+        """Attempt to retrieve non-existent plugin, return None"""
+
+        ploader = loader.PluginLoader(group='testdata', library='tests.testdata.lib')
+        self.assertIsNone(ploader.get_plugin('penguin', 'json'))
+        self.assertIsNone(ploader.get_plugin('parser', 'penguin'))
+
+        ploader = loader.PluginLoader(group='testdata', library='tests.testdata.lib')
+        self.assertIsNone(ploader.get_plugin('parser', 'json', '300.1.1'))
+
+    def test_get_plugin_filtered(self):
+        """Retrieve specific plugin if not filtered"""
+
+        ploader = loader.PluginLoader(group='testdata', library='tests.testdata.lib',
+                                      type_filter=('engine', 'hook'))
+
+        self.assertIsNone(ploader.get_plugin('parser', 'json'))
+
+        steamplugin = ploader.get_plugin('engine', 'steam')
+        self.assertEqual(steamplugin.name, 'steam')
+
+    def test_get_plugin_blacklist(self):
+        """Retrieve specific plugin if not blacklisted"""
+
+        blacklist = [('parser', 'json', '2.0'), ('parser', 'xml'), ('engine',)]
+
+        ploader = loader.PluginLoader(group='testdata', library='tests.testdata.lib',
+                                      blacklist=blacklist)
+
+        self.assertIsNone(ploader.get_plugin('parser', 'json', '2.0'))
+        self.assertIsNone(ploader.get_plugin('parser', 'xml'))
+        self.assertIsNone(ploader.get_plugin('engine', 'steam'))
+
+        jsonplugin = ploader.get_plugin('parser', 'json')
+        self.assertEqual(jsonplugin.name, 'json')
+        self.assertEqual(jsonplugin.version, '1.0')
